/*

  ---------------------					   -------
  Adding MP3-Tags (ID1) to files which are administered by CDMagic (www.cdmagic.de)
  ---------------------					   -------

  For use in a MediaTomb-environment, in order to have complete information about a MP3-file,
  before adding it to the MediaTomb-database.


  Presumed:
  All MP3-files are in a directory-structure like:

  x:\CDMagic\MP3\<artist>\<album>\title.mp3

  Where:
  x = driveletter, one of D, E, F, ...

  This Rexx-script must be run for each drive on which there are mp3-files
  according to the directory structure mentioned above.
  Before each run you must modify the variable named 'MP3Path'.


  How does it work?
  -----------------
  Normally, a program has no knowledge about file-properties like 'Artist', 'Album', 'Genre' or 'Year'
  even the 'Title' may not neccessarily be deducted from the filename. Therefore are programm cannot maintain
  ID-Tags for a mp3-file automatically.

  The only chance is to deduct these tags from the path-name of the file, provided the files are stored in a
  directory-structure with 'meaningful' names. This is the case with CDMagic, the MP3-database-program, which stores
  all kind of information about a song in its own database, and maintaining a strict correlation between
  database-fields 'Artist', 'Album' and 'Title' and the corresponding name of the file and its directory-names.

  In CDMagic, a MP3-file is stored with a path-name of x:\CDMagic\MP3\<artist>\<album>\title.mp3, allowing to deduct
  from the fully qualified filename the relevant ID-tags for 'Artist', 'Album', 'Title', and even 'Year' (which is
  taken from the file's timestamp, which may be incorrect, but still better as no information at all).

  What remains impossible to resolve is the 'Genre', where absolutely no hint at all may be derived from just
  the path and filename of an MP3-file. So here it is left untouched, except for the cases, where some eyecatcher-
  words are found in the path / filename: 'Jazz' and 'Blues'.

  Important:
  ----------
  If a certain tag in a file is found to have valid content, then this script will leave this tag untouched.


  Does this script have any value for non-CDMagic-users ?
  -------------------------------------------------------
  It depends. If the user has no 'meaningful' structure applied for his/her MP3-files, then this script may
  turn out to be useless.

  But, on the other hand, if the MP3-files are stored in directories which in some way have 'meaningful' names
  then this script may me imployed, but of course: after some modifications have been made.

  If, for instance your files are stored with filenames such as:

  "Wilson Pickett - In the midnight hour (Best of Rhythm & Blues).mp3"

  then you could deduct the relevant data by statements like this:

  parse value filename with Artist "-" Title "." garbage;
  parse value Title with Title "(" Album ")" garbage;


*/


MP3Path = "K:\CDMagic\MP3";		/* without trailing '\'				*/

No_Album = "Single (no album)";		/* Use this tag where no album may be deducted	*/

/* ===========		*/
/* Here we go: 		*/
/* ===========		*/

ok = SysFileTree( MP3Path"\*.mp3", "files.", "FS" );
do i = 1 to files.0

   FileSpec = subword(files.i, 5);

   ok = Get_MP3Tags( FileSpec );

   if Tags.Titel  <> "" 	,
    & Tags.Artist <> ""	,
    & Tags.Album  <> ""	,
    & Tags.Year   <> ""	then ITERATE;

   say "";
   Call XSay "'" || filespec("name",FileSpec) || "' has incomplete TAGs and will be processed:";

   Call DisplayTags;	

   AuxPath = right(FileSpec, length(FileSpec) - (length(MP3Path)+1) ); 	/* strip MP3Path	*/

   /* ---------------------------		*/
   /* At first: deduct from path:		*/
   /* ---------------------------		*/
   parse value AuxPath with !Artist "\" !Album "\" !Titel;

   if !Titel = "" & !Album  = "" then do
      !Titel  = !Artist;
      !Artist = "unbekannt";
      !Album  = "Einzeltitel";
   end;

   if !Titel = "" & !Album <> "" then do
      !Titel = !Album;
      if TAGs.Album <> "" then !Album = TAGs.Album; else; !Album = No_Album;
   end;

   if translate(right(!Titel,4)) = ".MP3" then 
      !Titel = left(!Titel, length(!Titel)-4);


   !Year = word(files.i,1);
   parse value !Year with mm "/" dd "/" yy;
   if yy < 10 then !Year = 2000 + yy;
   else;	      !Year = 1900 + yy;

   !Genre   = "12"x;		/* other		*/
   !Comment = "TAGs maintained by CDM_SetTags";

   if pos( "BLUES", translate(TAGs.Titel||TAGs.Artist||TAGs.Album) ) > 0 then
      !Genre = "00"x;	/* Blues		*/

   if pos( "JAZZ", translate(TAGs.Titel||TAGs.Artist||TAGs.Album) ) > 0 then
      !Genre = "08";		/* Jazz			*/


   /* ---------------------------------	*/
   /* Danach: Nimm aus TAGs was da ist:	*/
   /* ---------------------------------	*/
   if TAGs.Titel   = "" then TAGs.Titel   = !Titel;
   if TAGs.Artist  = "" then TAGs.Artist  = !Artist;
   if TAGs.Album   = "" then TAGs.Album   = !Album;
   if TAGs.Year    = "" then TAGs.Year    = !Year;
   if TAGs.Comment = "" then TAGs.Comment = !Comment;
   if TAGs.Genre   = "" then TAGs.Genre   = !Genre;

   Call XSay "===== Setze Neue Tags: ================================";
   Call DisplayTags;

   Call InsertTags FileSpec;
      
   if(0) then do;
      ok = beep(222,333);
      say "press any key ..."; pull;
   end;

end;
say files.0 "files processed";

exit;



/* ---------------			*/
/* Read MP3-Tags:			*/
/* ---------------			*/
Get_MP3Tags: procedure Expose Tags.
FileSpec = arg(1);

ok = SysFileTree( FileSpec, "files.", "F" );
if files.0 = 0 then RETURN -1;

Size = word(files.1,3);

if (0) then say "FileSpec="FileSpec",Size="Size"!";

LenTags = 128;
TAGs = charin(FileSpec, Size-LenTags, LenTags),

ok = stream(FileSpec, "c","close");

Posi = pos("TAG", TAGs)
if Posi = 0 then do
   TAGs = left("TAG", 200, " ");        /* gib einen Blank-String           */
   Posi = 1;
end;

Posi = Posi +3;  /* skip "TAG"          */

TAGs = substr(TAGs,Posi, LenTags-3);    /* TAGs pur (direkt hinter 'TAG')   */
TAGs = translate(TAGs, " ", "00"x);     /* Mach Blanks aus xNull            */

Tags. = "";
Tags.TagPos = (Size-LenTags) + Posi+3;	/* Hinter 'TAG'	*/
Tags.Titel  = strip( substr(TAGs,  1, 30) );
Tags.Artist = strip( substr(TAGs, 31, 30) );
Tags.Album  = strip( substr(TAGs, 61, 30) );
Tags.Year   = strip( substr(TAGs, 91,  4) );
Tags.Comment= strip( substr(TAGs, 95, 30) );    /* not used                 */
Tags.Genre  = substr(TAGs, 125, 1 );

return 1;	/* Tags. is set		*/

/* ------------------------------       */
/* Insert TAGs:				*/
/* ------------------------------       */
InsertTags: Procedure Expose TAGs.
FileSpec = arg(1);

TagArea = "TAG"	,
     ||   left(TAGs.Titel,  30," ")	,
     ||   left(TAGs.Artist, 30," ")	,
     ||   left(TAGs.Album,  30," ")	,
     ||   left(TAGs.Year,    4," ")	,
     ||   left(TAGs.Comment,30," ")	,
     ||   left(TAGs.Genre,   1);


/* -----------------------------------------------------------	*/
/* Kill all non-printable chars (lower than "21"):		*/
/* -----------------------------------------------------------	*/
do i = 1 to length(TagArea)
   Byte = substr(TagArea,i,1);
   MisPos = verify(Byte, "000102030405060708091011121314151617181920"x );
   if MisPos = 0 then
      TagArea = overlay(" ",TagArea, i);
end;

FLen = Chars( FileSpec );

Leng_ID3 = "7F"x;
Offs_ID3 = FLen - c2d(Leng_ID3);

ok = CharOut( FileSpec, TagArea, Offs_ID3 );

ok = stream(FileSpec, "c","close");


return 0;

/* ------------------------------       */
/* Display TAGs:			*/
/* ------------------------------       */
DisplayTags: Procedure Expose TAGs.

Call XSay left("",60,"-")		||"-"
Call XSay left("Pos",20,".")		||":"||Tags.TagPos;
Call XSay left("Titel",20,".")		||":"||Tags.Titel;
Call XSay left("Artist",20,".")		||":"||Tags.Artist;
Call XSay left("Album",20,".")		||":"||Tags.Album;
Call XSay left("Year",20,".")		||":"||Tags.Year;
Call XSay left("Comment",20,".")	||":"||Tags.Comment;
Call XSay left("Genre",20,".")		||":"||Tags.Genre;

return 0;

/* ------------------------------       */
/* XSay:              			*/
/* ------------------------------       */
XSay: procedure
What = arg(1);
Aux  = arg(2);

if strip(What) = "" | What="TOF" | What="BOF" then return;

if Aux <> "" then
   What = Aux||What;

What = time()"."substr(time("LONG"),10,2)"> "What;

say What;

return 0;

